//
// 7SEGMENT LED x 4 driver
//
// Copyright(c) 2002 Tatsuyuki Satoh , All rights reserved
//
// Important !
//
// 1. License
//
// This program is freeware for non-commercial use. 
// The user of this source must assume all legal 
// responsibility in the use of this program. 
// This is not GPL.
//
// 2. Guarantee 
//
// An author does no guarantee about this program.
// You can use this under your own risk. 
// 
// 3. A rule to distribute.
//   Do either 1) or 2).
//   1)Include a perfect original source.
//   2)Include all these headers , and add document of 
//     modification points
//

// 16bit number to 7segment conversion
module hex4_led(HEX4,DOT,CNT,EN,SEG,COM);
input [15:0] HEX4;
input [3:0] DOT;
input [1:0] CNT;
input EN;
output [7:0] SEG;
output [3:0] COM;

wire [3:0] HEX1;
wire DOT1;

// segment decoder
function [7:0] seg_dec;
input [3:0] hex;
input dot;
input en;
begin
if( en ) begin
  case ( hex )
     0: seg_dec[6:0] = 7'b1000000;
     1: seg_dec[6:0] = 7'b1111001;
     2: seg_dec[6:0] = 7'b0100100;
     3: seg_dec[6:0] = 7'b0110000;
     4: seg_dec[6:0] = 7'b0011001;
     5: seg_dec[6:0] = 7'b0010010;
     6: seg_dec[6:0] = 7'b0000010;
     7: seg_dec[6:0] = 7'b1111000;
     8: seg_dec[6:0] = 7'b0000000;
     9: seg_dec[6:0] = 7'b0010000;
    10: seg_dec[6:0] = 7'b0001000;
    11: seg_dec[6:0] = 7'b0000011;
    12: seg_dec[6:0] = 7'b1000110;
    13: seg_dec[6:0] = 7'b0100001;
    14: seg_dec[6:0] = 7'b0000110;
    15: seg_dec[6:0] = 7'b0001110;
  endcase
  seg_dec[7] = ~dot;
end else
  seg_dec = 8'b11111111;
end
endfunction

// common decoder
function [3:0] com_dec;
input [1:0] cnt;
input en;
begin
if( en )
case ( cnt )
     0: com_dec = 4'b1110;
     1: com_dec = 4'b1101;
     2: com_dec = 4'b1011;
     3: com_dec = 4'b0111;
  endcase
else
  com_dec = 4'b1111;
end
endfunction

// ZN^[
// RfR[_[
function [3:0] hex_sel;
input [15:0] hex4;
input [1:0] cnt;
begin
case ( cnt )
     0: hex_sel = hex4[3:0];
     1: hex_sel = hex4[7:4];
     2: hex_sel = hex4[11:8];
     3: hex_sel = hex4[15:12];
  endcase
end
endfunction

function dot_sel;
input [3:0] dot4;
input [1:0] cnt;
begin
case ( cnt )
     0: dot_sel = dot4[0];
     1: dot_sel = dot4[1];
     2: dot_sel = dot4[2];
     3: dot_sel = dot4[3];
  endcase
end
endfunction

assign HEX1 = hex_sel(HEX4,CNT);
assign DOT1 = dot_sel(DOT,CNT);
assign SEG  = seg_dec(HEX1,DOT1,EN);
assign COM  = com_dec(CNT,EN );

endmodule
